"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageService = void 0;
class LanguageService {
    constructor() {
        this._wrappers = [];
    }
    decorate(languageService) {
        const intercept = Object.create(null);
        for (const { name, wrapper } of this._wrappers) {
            intercept[name] = wrapper(languageService[name].bind(languageService));
        }
        return new Proxy(languageService, {
            get: (target, property) => {
                return intercept[property] || target[property];
            },
        });
    }
    wrap(name, wrapper) {
        this._wrappers.push({ name, wrapper });
        return this;
    }
    translateTextSpan(context, span) {
        return {
            start: context.node.getStart() + 1 + span.start,
            length: span.length,
        };
    }
    translateDefinitionInfo(context, definition) {
        return Object.assign(Object.assign({}, definition), { fileName: definition.fileName || context.fileName, textSpan: this.translateTextSpan(context, definition.textSpan) });
    }
}
exports.LanguageService = LanguageService;
//# sourceMappingURL=LanguageService.js.map