"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemixProject = void 0;
const events_1 = require("events");
const path = __importStar(require("path"));
const helpers_1 = require("./helpers");
const module_1 = require("./module");
const REMIX_RUNTIMES = [
    '@remix-run/node',
    '@remix-run/deno',
    '@remix-run/cloudflare',
];
class RemixProject extends events_1.EventEmitter {
    constructor(logger, project) {
        super();
        this.logger = logger;
        this.project = project;
        this.routes = [];
        this.root = project.getCurrentDirectory();
        this.runtime = this.detectRuntime();
    }
    async load() {
        let readConfig;
        try {
            readConfig = (0, module_1.requireModule)('@remix-run/dev/config', this.root).readConfig;
        }
        catch (e) {
            readConfig = (0, module_1.requireModule)('@remix-run/dev/dist/config', this.root).readConfig;
        }
        this.config = await readConfig(this.root);
        this.routes = this.loadRoutes();
    }
    ready() {
        return !!this.config;
    }
    loadRoutes() {
        const routesInfo = [];
        const handleRoutesRecursive = (parentId, parentPath = []) => {
            let routes = Object.values(this.config.routes).filter((route) => route.parentId === parentId);
            routes.forEach((route) => {
                let currentPath = parentPath;
                if (route.id !== 'root') {
                    currentPath = [...currentPath, route];
                    const fullPath = currentPath.reduce((acc, curr) => [acc, (0, helpers_1.trimSlash)(curr.path || '')].join('/'), '');
                    routesInfo.push({
                        path: fullPath,
                        fileName: path.join(this.config.appDirectory, route.file),
                        relativeFileName: route.file,
                        params: this.parse(currentPath),
                    });
                }
                handleRoutesRecursive(route.id, currentPath);
            });
        };
        handleRoutesRecursive();
        return routesInfo;
    }
    watch() {
        const addRoot = this.project.addRoot;
        this.project.addRoot = (...args) => {
            this.load();
            addRoot.apply(this.project, args);
        };
        const removeFile = this.project.removeFile;
        this.project.removeFile = (...args) => {
            this.load();
            removeFile.apply(this.project, args);
        };
        const configFile = this.project.getScriptInfo(path.join(this.project.getCurrentDirectory(), 'remix.config.js'));
        if (configFile) {
            const editContent = configFile.editContent;
            configFile.editContent = (...args) => {
                editContent.apply(configFile, args);
            };
        }
    }
    findRoutesByText(text) {
        return this.routes.filter(({ path }) => {
            const routeSegments = path.split('/');
            const textSegments = text.split('/');
            if (routeSegments.length !== textSegments.length) {
                return false;
            }
            const length = routeSegments.length > textSegments.length
                ? routeSegments.length
                : textSegments.length;
            for (let i = 0; i < length; i++) {
                const routeSegment = routeSegments[i];
                const textSegment = textSegments[i];
                if (routeSegment.startsWith(':')) {
                    if (!textSegment.startsWith(':')) {
                        return false;
                    }
                }
                else if (routeSegment !== textSegment) {
                    return false;
                }
            }
            return true;
        });
    }
    findRoutesByFile(fileName) {
        return this.routes.filter((route) => route.fileName == fileName);
    }
    detectRuntime() {
        const pkg = require(path.join(this.root, 'package.json'));
        for (const dep of Object.keys(pkg.dependencies)) {
            if (REMIX_RUNTIMES.includes(dep)) {
                return {
                    name: dep,
                    typings: (0, module_1.resolveTypings)(dep, this.root),
                };
            }
        }
    }
    parse(routes) {
        const paramNames = [];
        routes.forEach((route) => {
            return (route.path &&
                paramNames.push(...route.path
                    .split('/')
                    .filter((seg) => seg.startsWith(':'))
                    .map((param) => param.replace(':', ''))));
        });
        return paramNames;
    }
}
exports.RemixProject = RemixProject;
//# sourceMappingURL=RemixProject.js.map