"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RouteHelperLanguageService = void 0;
const path_1 = __importDefault(require("path"));
const ts = __importStar(require("typescript/lib/tsserverlibrary"));
const LanguageService_1 = require("./LanguageService");
const RouteContext_1 = require("./RouteContext");
const slash_1 = __importDefault(require("slash"));
class RouteHelperLanguageService extends LanguageService_1.LanguageService {
    constructor(tsProject, remixProject, sourceHelper) {
        super();
        this.tsProject = tsProject;
        this.remixProject = remixProject;
        this.sourceHelper = sourceHelper;
        this.tryAdaptGetQuickInfoAtPosition();
        this.tryAdaptGetDefinitionAndBoundSpan();
    }
    tryAdaptGetQuickInfoAtPosition() {
        this.wrap('getQuickInfoAtPosition', (delegate) => (fileName, position) => {
            const context = this.getRoute(fileName, position);
            if (!context) {
                return delegate(fileName, position);
            }
            const routes = this.remixProject.findRoutesByText(context.rawText);
            return {
                kind: ts.ScriptElementKind.string,
                kindModifiers: '',
                displayParts: routes.slice(0, 1).map((route) => ({
                    text: `(remix) file: ${(0, slash_1.default)(path_1.default.join('~/app', route.relativeFileName))}`,
                    kind: ts.SymbolDisplayPartKind.methodName,
                })),
                textSpan: {
                    start: 0,
                    length: context.node.getFullWidth(),
                },
            };
        });
    }
    tryAdaptGetDefinitionAndBoundSpan() {
        this.wrap('getDefinitionAndBoundSpan', (delegate) => (fileName, position, ...rest) => {
            const context = this.getRoute(fileName, position);
            if (context) {
                const routes = this.remixProject.findRoutesByText(context.rawText);
                const textSpan = {
                    start: 0,
                    length: 0,
                };
                return {
                    definitions: routes.map((route) => ({
                        name: route.fileName,
                        kind: ts.ScriptElementKind.string,
                        containerKind: ts.ScriptElementKind.unknown,
                        containerName: '',
                        fileName: route.fileName,
                        textSpan,
                    })),
                    textSpan,
                };
            }
            return delegate(fileName, position, ...rest);
        });
    }
    getRoute(fileName, position) {
        const node = this.getValidRouteNode(this.sourceHelper.getNode(fileName, position));
        if (!node) {
            return undefined;
        }
        // Make sure we are inside the route string
        if (position <= node.pos) {
            return undefined;
        }
        return new RouteContext_1.RouteContext(fileName, node);
    }
    getValidRouteNode(node) {
        var _a, _b;
        if (!node) {
            return undefined;
        }
        if (node.kind !== ts.SyntaxKind.StringLiteral) {
            return undefined;
        }
        const parentExpressionText = (_b = (_a = node.parent) === null || _a === void 0 ? void 0 : _a.expression) === null || _b === void 0 ? void 0 : _b.getText();
        if (parentExpressionText !== '$path' &&
            parentExpressionText !== '$params') {
            return undefined;
        }
        return node;
    }
}
exports.RouteHelperLanguageService = RouteHelperLanguageService;
//# sourceMappingURL=RouteHelperLanguageService.js.map