"use strict";
const LanguageServiceLogger_1 = require("./LanguageServiceLogger");
const RemixProject_1 = require("./RemixProject");
const RouteHelperLanguageService_1 = require("./RouteHelperLanguageService");
const ScriptSourceHelper_1 = require("./ScriptSourceHelper");
function init(modules) {
    const ts = modules.typescript;
    function create(info) {
        const logger = new LanguageServiceLogger_1.LanguageServiceLogger(info);
        // Set up decorator object
        let proxy = Object.create(null);
        for (let k of Object.keys(info.languageService)) {
            logger.log(k);
            const x = info.languageService[k];
            // @ts-expect-error - JS runtime trickery which is tricky to type tersely
            proxy[k] = (...args) => x.apply(info.languageService, args);
        }
        const remixProject = new RemixProject_1.RemixProject(logger, info.project);
        if (!remixProject.runtime) {
            throw new Error('Can not defect remix runtime');
        }
        remixProject.load();
        remixProject.watch();
        const routeHelperLanguageService = new RouteHelperLanguageService_1.RouteHelperLanguageService(info.project, remixProject, new ScriptSourceHelper_1.ScriptSourceHelper(ts, info.project));
        return routeHelperLanguageService.decorate(proxy);
    }
    return { create };
}
module.exports = init;
//# sourceMappingURL=index.js.map